/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2006-7 H. Jasak All rights reserved
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    timeFunctionGgiFvMesh


Author
    Hrvoje Jasak, Wikki Ltd.  All rights reserved.
    Fethi Tekin, All rights reserved.
    Oliver Borm, All rights reserved.
    Gabriel Bousquet, All rights reserved.
SourceFiles
    timeFunctionGgiFvMesh.C

\*---------------------------------------------------------------------------*/

#ifndef timeFunctionGgiFvMesh_H
#define timeFunctionGgiFvMesh_H

#include "dynamicFvMesh.H"
#include "cylindricalCS.H"


// from subset
#include "fvMeshSubset.H"


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

  // from subset
  class motionSolver;

/*---------------------------------------------------------------------------*\
                        Class timeFunctionGgiFvMesh Declaration
\*---------------------------------------------------------------------------*/

class timeFunctionGgiFvMesh
:
    public dynamicFvMesh
{
    // Private data

        //- Motion dictionary
        dictionary dict_;

        //- Coordinate system
        autoPtr<coordinateSystem> csPtr_;

        //- Markup field for points.
        mutable vectorField* movingPointsPtr_;

        //- Markup field for points.  Ring points marked with 1
        mutable scalarField* movingRingPointsMaskPtr_;

  // From subset
        //- Moving mesh dictionary
        // dictionary movingMeshCoeffs_;

        //- Subset mesh
        fvMeshSubset subsetMesh_;

        //- Pointer to mesh motion
        autoPtr<motionSolver> motionPtr_;


  
    // Private Member Functions

        //- Disallow default bitwise copy construct
        timeFunctionGgiFvMesh(const timeFunctionGgiFvMesh&);

        //- Disallow default bitwise assignment
        void operator=(const timeFunctionGgiFvMesh&);

        //- Calculate moving Points
        void calcMovingPoints() const;

        //- Return moving points
        const vectorField& movingPoints() const;

        //- Return moving points Mask
        const scalarField& movingRingPointsMask() const;

        //- Calculate moving masks
        void calcMovingRingMask() const;

public:

    //- Runtime type information
    TypeName("timeFunctionGgiFvMesh");


    // Constructors

        //- Construct from IOobject
        explicit timeFunctionGgiFvMesh(const IOobject& io);


    // Destructor

        virtual ~timeFunctionGgiFvMesh();


    // Member Functions

        //- Return coordinate system
        const coordinateSystem& cs() const
        {
            return csPtr_();
        }

        //- Update the mesh for both mesh motion
        virtual bool update();


  // From subset

        //- Return reference to subset mesh
        const fvMeshSubset& subsetMesh() const;

};

#include "timeFunction.H"




// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
